@@ -1,13 +1,15 @@
 //! To speed up the tests, they are tests all collected in a single module
 
 use common::{
-    develop, errors, get_python_implementation, handle_result, integration, other, test_python_path,
+    develop, errors, handle_result, integration, other, test_python_implementation,
+    TestInstallBackend,
 };
 use expect_test::expect;
 use maturin::pyproject_toml::SdistGenerator;
-use maturin::Target;
+use rstest::rstest;
 use std::env;
-use std::path::{Path, PathBuf};
+use std::path::Path;
+use std::time::Duration;
 use time::macros::datetime;
 use which::which;
 
@@ -20,19 +22,19 @@ fn develop_pyo3_pure() {
         None,
         "develop-pyo3-pure",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
 #[test]
-#[ignore]
 fn develop_pyo3_pure_conda() {
-    // Only run on GitHub Actions for now
-    if env::var("GITHUB_ACTIONS").is_ok() {
+    if which("conda").is_ok() {
         handle_result(develop::test_develop(
             "test-crates/pyo3-pure",
             None,
             "develop-pyo3-pure-conda",
             true,
+            TestInstallBackend::Pip,
         ));
     }
 }
@@ -44,6 +46,7 @@ fn develop_pyo3_mixed() {
         None,
         "develop-pyo3-mixed",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
@@ -54,6 +57,7 @@ fn develop_pyo3_mixed_include_exclude() {
         None,
         "develop-pyo3-mixed-include-exclude",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
@@ -64,6 +68,18 @@ fn develop_pyo3_mixed_submodule() {
         None,
         "develop-pyo3-mixed-submodule",
         false,
+        TestInstallBackend::Pip,
+    ));
+}
+
+#[test]
+fn develop_pyo3_mixed_with_path_dep() {
+    handle_result(develop::test_develop(
+        "test-crates/pyo3-mixed-with-path-dep",
+        None,
+        "develop-pyo3-mixed-with-path-dep",
+        false,
+        TestInstallBackend::Pip,
     ));
 }
 
@@ -74,6 +90,7 @@ fn develop_pyo3_mixed_implicit() {
         None,
         "develop-pyo3-mixed-implicit",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
@@ -84,6 +101,7 @@ fn develop_pyo3_mixed_py_subdir() {
         None,
         "develop-pyo3-mixed-py-subdir",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
@@ -94,26 +112,39 @@ fn develop_pyo3_mixed_src_layout() {
         None,
         "develop-pyo3-mixed-src",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
 #[test]
 fn develop_cffi_pure() {
+    let python_implementation = test_python_implementation().unwrap();
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // TODO: PyPy hangs on cffi test sometimes
+        return;
+    }
     handle_result(develop::test_develop(
         "test-crates/cffi-pure",
         None,
         "develop-cffi-pure",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
 #[test]
 fn develop_cffi_mixed() {
+    let python_implementation = test_python_implementation().unwrap();
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // PyPy hangs on cffi test sometimes
+        return;
+    }
     handle_result(develop::test_develop(
         "test-crates/cffi-mixed",
         None,
         "develop-cffi-mixed",
         false,
+        TestInstallBackend::Pip,
     ));
 }
 
@@ -125,10 +156,22 @@ fn develop_uniffi_pure() {
             None,
             "develop-uniffi-pure",
             false,
+            TestInstallBackend::Pip,
         ));
     }
 }
 
+#[test]
+fn develop_uniffi_pure_proc_macro() {
+    handle_result(develop::test_develop(
+        "test-crates/uniffi-pure-proc-macro",
+        None,
+        "develop-uniffi-pure-proc-macro",
+        false,
+        TestInstallBackend::Pip,
+    ));
+}
+
 #[test]
 fn develop_uniffi_mixed() {
     if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
@@ -137,37 +180,81 @@ fn develop_uniffi_mixed() {
             None,
             "develop-uniffi-mixed",
             false,
+            TestInstallBackend::Pip,
         ));
     }
 }
 
 #[test]
-fn develop_hello_world() {
+fn develop_uniffi_multiple_binding_files() {
+    if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
+        handle_result(develop::test_develop(
+            "test-crates/uniffi-multiple-binding-files",
+            None,
+            "develop-uniffi-multiple-binding-files",
+            false,
+            TestInstallBackend::Pip,
+        ));
+    }
+}
+
+#[rstest]
+#[timeout(Duration::from_secs(60))]
+#[case(TestInstallBackend::Pip, "pip")]
+#[case(TestInstallBackend::Uv, "uv")]
+#[test]
+fn develop_hello_world(#[case] backend: TestInstallBackend, #[case] name: &str) {
+    // Only run uv tests on platforms that has wheel on PyPI or when uv binary is found
+    if matches!(backend, TestInstallBackend::Uv)
+        && !cfg!(any(
+            target_os = "linux",
+            target_os = "macos",
+            target_os = "windows"
+        ))
+        && which("uv").is_err()
+    {
+        return;
+    }
+
     handle_result(develop::test_develop(
         "test-crates/hello-world",
         None,
-        "develop-hello-world",
+        format!("develop-hello-world-{}", name).as_str(),
         false,
+        backend,
     ));
 }
 
+#[rstest]
+#[timeout(Duration::from_secs(120))]
+#[case(TestInstallBackend::Pip, "pip")]
+#[case(TestInstallBackend::Uv, "uv")]
 #[test]
-fn develop_pyo3_ffi_pure() {
+fn develop_pyo3_ffi_pure(#[case] backend: TestInstallBackend, #[case] name: &str) {
+    // Only run uv tests on platforms that has wheel on PyPI or when uv binary is found
+    if matches!(backend, TestInstallBackend::Uv)
+        && !cfg!(any(
+            target_os = "linux",
+            target_os = "macos",
+            target_os = "windows"
+        ))
+        && which("uv").is_err()
+    {
+        return;
+    }
+
     handle_result(develop::test_develop(
         "test-crates/pyo3-ffi-pure",
         None,
-        "develop-pyo3-ffi-pure",
+        format!("develop-pyo3-ffi-pure-{}", name).as_str(),
         false,
+        backend,
     ));
 }
 
 #[test]
 fn integration_pyo3_bin() {
-    let python = test_python_path().map(PathBuf::from).unwrap_or_else(|| {
-        let target = Target::from_target_triple(None).unwrap();
-        target.get_python()
-    });
-    let python_implementation = get_python_implementation(&python).unwrap();
+    let python_implementation = test_python_implementation().unwrap();
     if python_implementation == "pypy" || python_implementation == "graalpy" {
         // PyPy & GraalPy do not support the 'auto-initialize' feature of pyo3
         return;
@@ -226,6 +313,17 @@ fn integration_pyo3_mixed_submodule() {
     ));
 }
 
+#[test]
+fn integration_pyo3_mixed_with_path_dep() {
+    handle_result(integration::test_integration(
+        "test-crates/pyo3-mixed-with-path-dep",
+        None,
+        "integration-pyo3-mixed-with-path-dep",
+        false,
+        None,
+    ));
+}
+
 #[test]
 fn integration_pyo3_mixed_implicit() {
     handle_result(integration::test_integration(
@@ -262,8 +360,7 @@ fn integration_pyo3_mixed_src_layout() {
 #[test]
 #[cfg_attr(target_os = "macos", ignore)] // Don't run it on macOS, too slow
 fn integration_pyo3_pure_conda() {
-    // Only run on GitHub Actions for now
-    if env::var("GITHUB_ACTIONS").is_ok() {
+    if which("conda").is_ok() {
         handle_result(integration::test_integration_conda(
             "test-crates/pyo3-mixed",
             None,
@@ -273,6 +370,11 @@ fn integration_pyo3_pure_conda() {
 
 #[test]
 fn integration_cffi_pure() {
+    let python_implementation = test_python_implementation().unwrap();
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // PyPy hangs on cffi test sometimes
+        return;
+    }
     handle_result(integration::test_integration(
         "test-crates/cffi-pure",
         None,
@@ -284,6 +386,11 @@ fn integration_cffi_pure() {
 
 #[test]
 fn integration_cffi_mixed() {
+    let python_implementation = test_python_implementation().unwrap();
+    if env::var("GITHUB_ACTIONS").is_ok() && python_implementation == "pypy" {
+        // PyPy hangs on cffi test sometimes
+        return;
+    }
     handle_result(integration::test_integration(
         "test-crates/cffi-mixed",
         None,
@@ -306,6 +413,17 @@ fn integration_uniffi_pure() {
     }
 }
 
+#[test]
+fn integration_uniffi_pure_proc_macro() {
+    handle_result(integration::test_integration(
+        "test-crates/uniffi-pure-proc-macro",
+        None,
+        "integration-uniffi-pure-proc-macro",
+        false,
+        None,
+    ));
+}
+
 #[test]
 fn integration_uniffi_mixed() {
     if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
@@ -352,6 +470,28 @@ fn integration_with_data() {
     ));
 }
 
+#[test]
+fn integration_readme_duplication() {
+    handle_result(integration::test_integration(
+        "test-crates/readme-duplication/readme-py",
+        None,
+        "integration-readme-duplication",
+        false,
+        None,
+    ));
+}
+
+#[test]
+fn integration_workspace_inverted_order() {
+    handle_result(integration::test_integration(
+        "test-crates/workspace-inverted-order/path-dep-with-root",
+        None,
+        "integration-workspace-inverted-order",
+        false,
+        None,
+    ));
+}
+
 #[test]
 // Sourced from https://pypi.org/project/wasmtime/11.0.0/#files
 // update with wasmtime updates
@@ -375,15 +515,12 @@ fn integration_wasm_hello_world() {
         None,
         "integration-wasm-hello-world",
         false,
-        Some("wasm32-wasi"),
+        Some("wasm32-wasip1"),
     ));
 
-    let python = test_python_path().map(PathBuf::from).unwrap_or_else(|| {
-        let target = Target::from_target_triple(None).unwrap();
-        target.get_python()
-    });
-    let python_implementation = get_python_implementation(&python).unwrap();
-    let venv_name = format!("integration-wasm-hello-world-py3-wasm32-wasi-{python_implementation}");
+    let python_implementation = test_python_implementation().unwrap();
+    let venv_name =
+        format!("integration-wasm-hello-world-py3-wasm32-wasip1-{python_implementation}");
 
     // Make sure we're actually running wasm
     assert!(Path::new("test-crates")
@@ -400,7 +537,7 @@ fn integration_wasm_hello_world() {
 
 #[test]
 fn abi3_without_version() {
-    handle_result(errors::abi3_without_version())
+    handle_result(other::abi3_without_version())
 }
 
 #[test]
@@ -410,23 +547,18 @@ fn abi3_without_version() {
         target_os = "linux",
         target_env = "gnu",
         any(
-            target_arch = "i686",
+            target_arch = "x86",
             target_arch = "x86_64",
             target_arch = "aarch64",
             target_arch = "powerpc64",
-            target_arch = "powerpc64le",
             target_arch = "s390x",
-            target_arch = "armv7"
+            target_arch = "arm"
         )
     )),
     ignore
 )]
 fn pyo3_no_extension_module() {
-    let python = test_python_path().map(PathBuf::from).unwrap_or_else(|| {
-        let target = Target::from_target_triple(None).unwrap();
-        target.get_python()
-    });
-    let python_implementation = get_python_implementation(&python).unwrap();
+    let python_implementation = test_python_implementation().unwrap();
     if python_implementation == "cpython" {
         handle_result(errors::pyo3_no_extension_module())
     }
@@ -462,6 +594,40 @@ fn workspace_cargo_lock() {
     handle_result(other::test_workspace_cargo_lock())
 }
 
+#[test]
+fn workspace_members_beneath_pyproject_sdist() {
+    let cargo_toml = expect![[r#"
+        [workspace]
+        resolver = "2"
+        members = ["pyo3-mixed-workspace", "python/pyo3-mixed-workspace-py"]
+        "#]];
+    handle_result(other::test_source_distribution(
+        "test-crates/pyo3-mixed-workspace/rust/python/pyo3-mixed-workspace-py",
+        SdistGenerator::Cargo,
+        expect![[r#"
+            {
+                "pyo3_mixed_workspace-2.1.3/PKG-INFO",
+                "pyo3_mixed_workspace-2.1.3/pyproject.toml",
+                "pyo3_mixed_workspace-2.1.3/rust/Cargo.lock",
+                "pyo3_mixed_workspace-2.1.3/rust/Cargo.toml",
+                "pyo3_mixed_workspace-2.1.3/rust/pyo3-mixed-workspace/Cargo.toml",
+                "pyo3_mixed_workspace-2.1.3/rust/pyo3-mixed-workspace/src/lib.rs",
+                "pyo3_mixed_workspace-2.1.3/rust/python/pyo3-mixed-workspace-py/Cargo.toml",
+                "pyo3_mixed_workspace-2.1.3/rust/python/pyo3-mixed-workspace-py/src/lib.rs",
+                "pyo3_mixed_workspace-2.1.3/src/pyo3_mixed_workspace/__init__.py",
+                "pyo3_mixed_workspace-2.1.3/src/pyo3_mixed_workspace/python_module/__init__.py",
+                "pyo3_mixed_workspace-2.1.3/src/pyo3_mixed_workspace/python_module/double.py",
+                "pyo3_mixed_workspace-2.1.3/src/tests/test_pyo3_mixed.py",
+            }
+        "#]],
+        Some((
+            Path::new("pyo3_mixed_workspace-2.1.3/rust/Cargo.toml"),
+            cargo_toml,
+        )),
+        "sdist-workspace-members-beneath_pyproject",
+    ))
+}
+
 #[test]
 fn workspace_members_non_local_dep_sdist() {
     let cargo_toml = expect![[r#"
@@ -474,12 +640,16 @@ fn workspace_members_non_local_dep_sdist() {
         license = "MIT"
 
         [dependencies]
-        pyo3 = { version = "0.19.0", features = ["abi3-py37", "extension-module", "generate-import-lib"] }
+        pyo3 = { version = "0.24.0", features = [
+            "abi3-py37",
+            "extension-module",
+            "generate-import-lib",
+        ] }
 
         [lib]
         name = "pyo3_pure"
         crate-type = ["cdylib"]
-        "#]];
+    "#]];
     handle_result(other::test_source_distribution(
         "test-crates/pyo3-pure",
         SdistGenerator::Cargo,
@@ -539,8 +709,7 @@ fn lib_with_target_path_dep_sdist() {
         crate-type = ["cdylib"]
 
         [dependencies]
-        # Don't use the macros feature, which makes compilation much faster
-        pyo3 = { version = "0.19.0", default-features = false, features = ["extension-module"] }
+        pyo3 = { version = "0.24.0", features = ["extension-module"] }
 
         [target.'cfg(not(target_endian = "all-over-the-place"))'.dependencies]
         some_path_dep = { path = "../some_path_dep" }
@@ -599,6 +768,7 @@ fn pyo3_mixed_include_exclude_sdist() {
         SdistGenerator::Cargo,
         expect![[r#"
             {
+                "pyo3_mixed_include_exclude-2.1.3/.gitignore",
                 "pyo3_mixed_include_exclude-2.1.3/Cargo.lock",
                 "pyo3_mixed_include_exclude-2.1.3/Cargo.toml",
                 "pyo3_mixed_include_exclude-2.1.3/PKG-INFO",
@@ -628,6 +798,7 @@ fn pyo3_mixed_include_exclude_git_sdist_generator() {
         SdistGenerator::Git,
         expect![[r#"
             {
+                "pyo3_mixed_include_exclude-2.1.3/.gitignore",
                 "pyo3_mixed_include_exclude-2.1.3/Cargo.lock",
                 "pyo3_mixed_include_exclude-2.1.3/Cargo.toml",
                 "pyo3_mixed_include_exclude-2.1.3/PKG-INFO",
